import { HttpClient, HttpParams, HttpHeaders } from '@angular/common/http';
import { HttpErrorResponse, HttpResponse } from '@angular/common/http';
import { Injectable, Inject, Optional } from '@angular/core';
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/operator/shareReplay';
import 'rxjs/add/observable/throw';
import 'rxjs/add/operator/catch';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/do';
import { LocalStorageCacheService } from "./local-storage-cache.service";
import { HTTP_WRAPPER_CONST } from "./http-constant";
var HttpWrapperService = /** @class */ (function () {
    function HttpWrapperService(http, baseUrl) {
        this.http = http;
        this.baseUrl = baseUrl;
        if (this.baseUrl === null || this.baseUrl === undefined)
            this.baseUrl = "";
    }
    HttpWrapperService.prototype.setRelativeUrl = function (urlConetxt) {
        this.url = this.baseUrl + urlConetxt;
    };
    HttpWrapperService.prototype.get = function (url, httpParams, httpHeadres, isFullResponse) {
        if (httpParams === void 0) { httpParams = null; }
        if (httpHeadres === void 0) { httpHeadres = null; }
        if (isFullResponse === void 0) { isFullResponse = false; }
        var serverURL = (url) ? this.baseUrl + url : this.url;
        var addtionalValues = this.createOptional(httpParams, httpHeadres, isFullResponse);
        return this.http.get(serverURL, addtionalValues).
            shareReplay().
            catch(this.handleError);
    };
    HttpWrapperService.prototype.post = function (url, body, httpHeadres, isFullResponse) {
        if (httpHeadres === void 0) { httpHeadres = null; }
        if (isFullResponse === void 0) { isFullResponse = false; }
        var serverURL = (url) ? this.baseUrl + url : this.url;
        var addtionalValues = this.createOptional(null, httpHeadres, isFullResponse);
        return this.http
            .post(serverURL, body, addtionalValues)
            .shareReplay().catch(this.handleError);
    };
    HttpWrapperService.prototype.put = function (url, body, httpHeadres, isFullResponse) {
        if (httpHeadres === void 0) { httpHeadres = null; }
        if (isFullResponse === void 0) { isFullResponse = false; }
        var serverURL = (url) ? this.baseUrl + url : this.url;
        var addtionalValues = this.createOptional(null, httpHeadres, isFullResponse);
        return this.http.put(serverURL, body, addtionalValues)
            .shareReplay().catch(this.handleError);
    };
    HttpWrapperService.prototype.delete = function (url, httpParams, httpHeadres, isFullResponse) {
        if (httpParams === void 0) { httpParams = null; }
        if (httpHeadres === void 0) { httpHeadres = null; }
        if (isFullResponse === void 0) { isFullResponse = false; }
        var serverURL = (url) ? this.baseUrl + url : this.url;
        var addtionalValues = this.createOptional(httpParams, httpHeadres, isFullResponse);
        return this.http.delete(this.baseUrl + url, addtionalValues).shareReplay().catch(this.handleError);
    };
    HttpWrapperService.prototype.patch = function (url, body, httpHeadres, isFullResponse) {
        if (isFullResponse === void 0) { isFullResponse = false; }
        var serverURL = (url) ? this.baseUrl + url : this.url;
        var addtionalValues = this.createOptional(null, httpHeadres, isFullResponse);
        return this.http.patch(serverURL, body, addtionalValues).shareReplay().catch(this.handleError);
    };
    HttpWrapperService.prototype.createOptional = function (httpParams, httpHeadres, isFullResponse) {
        if (isFullResponse === void 0) { isFullResponse = false; }
        var addtionalValues = {};
        addtionalValues["reportProgress"] = true;
        if (httpParams)
            addtionalValues["params"] = this.createHttpPrams(httpParams);
        if (httpHeadres)
            addtionalValues["headers"] = this.createHttpHeader(httpHeadres);
        if (isFullResponse)
            addtionalValues["observe"] = "response";
        return addtionalValues;
    };
    HttpWrapperService.prototype.createHttpPrams = function (params) {
        if (params instanceof HttpParams)
            return params;
        var httpParams = new HttpParams();
        Object.keys(params).forEach(function (key) {
            httpParams = httpParams.append(key, params[key]);
        });
        return httpParams;
    };
    HttpWrapperService.prototype.createHttpHeader = function (header) {
        if (header instanceof HttpHeaders)
            return header;
        return new HttpHeaders(header);
    };
    HttpWrapperService.prototype.handleError = function (err) {
        var errorMessage = '';
        if (err.error instanceof Error) {
            errorMessage = "An error occurred: " + err.error.message;
        }
        else {
            if (err instanceof HttpErrorResponse) {
                console.log('interceptor error');
                console.log(err);
                errorMessage = "Server returned code: " + err.status + ", error message is: " + err.message;
                if (err.status === 401) {
                    // Token  expired, can be setted to go to login
                    return Observable.throw(err);
                }
            }
        }
        return Observable.throw(errorMessage);
    };
    HttpWrapperService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    HttpWrapperService.ctorParameters = function () { return [
        { type: HttpClient, },
        { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [HTTP_WRAPPER_CONST.API_BASE_URL,] },] },
    ]; };
    return HttpWrapperService;
}());
export { HttpWrapperService };
var AppInterceptor = /** @class */ (function () {
    function AppInterceptor(localStorageCacheService, tokenKey, tokenStorage, headerKey, prefixKey, logTime) {
        this.localStorageCacheService = localStorageCacheService;
        this.tokenKey = tokenKey;
        this.tokenStorage = tokenStorage;
        this.headerKey = headerKey;
        this.prefixKey = prefixKey;
        this.logTime = logTime;
    }
    AppInterceptor.prototype.getTokenKey = function () {
        if (this.tokenStorage !== null && this.tokenStorage === undefined && this.tokenStorage === 'session')
            return sessionStorage.getItem(this.tokenKey);
        else
            return localStorage.getItem(this.tokenKey);
    };
    AppInterceptor.prototype.intercept = function (req, next) {
        var _this = this;
        var started = Date.now();
        if (this.tokenKey && this.getTokenKey() !== null) {
            if (this.prefixKey === null || this.prefixKey === undefined)
                this.prefixKey = "";
            req = req.clone({ headers: req.headers.set(this.headerKey, this.prefixKey + this.getTokenKey()) });
        }
        if (!req.headers.has(HTTP_WRAPPER_CONST.CONTENT_TYPE)) {
            req = req.clone({ headers: req.headers.set(HTTP_WRAPPER_CONST.CONTENT_TYPE, HTTP_WRAPPER_CONST.JSON_MIME) });
        }
        if (!req.headers.has(HTTP_WRAPPER_CONST.ACCEPT)) {
            req = req.clone({ headers: req.headers.set(HTTP_WRAPPER_CONST.ACCEPT, HTTP_WRAPPER_CONST.JSON_MIME) });
        }
        return next.handle(req).do(function (event) {
            if (event instanceof HttpResponse) {
                if (req.headers.has(HTTP_WRAPPER_CONST.CACHED_CONTENT))
                    _this.localStorageCacheService.put(req, event);
                var elapsed = Date.now() - started;
                if (_this.logTime)
                    console.log("Request for " + req.urlWithParams + " took " + elapsed + " ms.");
            }
        }, function (err) {
            if (err instanceof HttpErrorResponse) {
                var elapsed = Date.now() - started;
                if (_this.logTime)
                    console.log("Request for " + req.urlWithParams + " failed after " + elapsed + " ms.");
            }
        });
    };
    AppInterceptor.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    AppInterceptor.ctorParameters = function () { return [
        { type: LocalStorageCacheService, },
        { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [HTTP_WRAPPER_CONST.TOKEN_KEY,] },] },
        { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [HTTP_WRAPPER_CONST.TOKEN_STORAGE,] },] },
        { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [HTTP_WRAPPER_CONST.TOKEN_HEADER_KEY,] },] },
        { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [HTTP_WRAPPER_CONST.PREFIX_TOKEN_KEY,] },] },
        { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [HTTP_WRAPPER_CONST.PREFIX_TOKEN_KEY,] },] },
    ]; };
    return AppInterceptor;
}());
export { AppInterceptor };
//# sourceMappingURL=http-wrapper.service.js.map